/************************************************************************
 *
 *  PresentationStyleConverter.java
 *
 *  Copyright: 2002-2014 by Henrik Just
 *
 *  This file is part of Writer2LaTeX.
 *  
 *  Writer2LaTeX is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Writer2LaTeX is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Writer2LaTeX.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  Version 1.4 (2014-09-16)
 *
 */

package w2phtml.xhtml.style;

import java.util.Enumeration;
import java.util.Hashtable;

import w2phtml.office.OfficeReader;
import w2phtml.office.OfficeStyleFamily;
import w2phtml.office.StyleWithProperties;
import w2phtml.util.CSVList;
import w2phtml.util.ExportNameCollection;
import w2phtml.xhtml.Converter;
import w2phtml.xhtml.StyleInfo;
import w2phtml.xhtml.XhtmlConfig;
import w2phtml.xhtml.XhtmlStyleMap;
import w2phtml.xhtml.style.properties.Properties;

/**
 * This class converts OpenDocument presentation styles to CSS2 styles.
 * Presentation styles are special frame styles, used to style the standard
 * elements in a presentation (title, subtitle and textbox)
 */
public class PresentationStyleParser extends FrameStyleParser {

    // Data about outline styles
    String sCurrentOutlineStyle = null;
    Hashtable<String, String[]> outlineStyles = new Hashtable<String, String[]>();
    ExportNameCollection outlineStyleNames = new ExportNameCollection(true);

    /** Create a new <code>PresentationStyleConverter</code>
     *  @param ofr an <code>OfficeReader</code> to read style information from
     *  @param config the configuration to use
     *  @param converter the main <code>Converter</code> class
     *  @param nType the type of xhtml to use
     */
    public PresentationStyleParser(OfficeReader ofr, XhtmlConfig config, Converter converter, int nType) {
        super(ofr,config,converter,nType);
        // style maps for presentations are currently not supported:
        this.styleMap = new XhtmlStyleMap();
    }

    /** Return a prefix to be used in generated css class names
     *  @return the prefix
     */
    public String getClassNamePrefix() { return "prsnt"; }

    /** Get the family of presentation styles
     *  @return the style family
     */
    public OfficeStyleFamily getStyles() {
        return ofr.getPresentationStyles();
    }
	
    /** Create default tag name to represent a presentation object
     *  @param style to use
     *  @return the tag name.
     */
    public String getStyleTag() {
        return "div";
    }
	
    /** <p>Convert style information for used styles</p>
     *  @param sIndent a String of spaces to add before each line
     */
  	public String composeStyleDeclarations() {
 
  		StringBuilder buf = new StringBuilder();
  		buf.append(super.composeStyleDeclarations());
  		Enumeration<String> names = outlineStyleNames.keys();
  		while (names.hasMoreElements()) {
  			String displayName = names.nextElement();
  			buf.append(composeDeclaration(displayName));
  		}
  		return buf.toString();
  
  	}

		public String composeDeclaration(String displayName) {
			String declaration = "";
			StyleWithProperties style = (StyleWithProperties) getStyles().getStyleByDisplayName(displayName);
			String declarationBlock = composeDeclarationBlock(displayName);
			String selector = composeSelector(displayName);;
			if (!style.isAutomatic() && !selector.isEmpty() && !declarationBlock.isEmpty()) {
				// Apply style to paragraphs within a list item with this class
					declaration = createDeclaration(selector, declarationBlock);
			}
			return declaration;
		}

		private String composeSelector(String displayName) {
			styleNames.addName(displayName);
			return "li.outline" + styleNames.getName(displayName) + " p ";
		}

		public String composeDeclarationBlock(String displayName) {
			StyleWithProperties style = (StyleWithProperties) getStyles().getStyleByDisplayName(displayName);
			Properties props = new Properties(";");
			getFrameSP().cssMargins(style, props, true);
			getParSP().cssPar(style, props, true);
			getTextSP().cssTextCommon(style, props, true);
			return props.toString();
		}

	
    public void enterOutline(String sStyleName) {
        sCurrentOutlineStyle = sStyleName;
        if (!outlineStyles.containsKey(sCurrentOutlineStyle)) {
            String[] sNames = new String[10];
            outlineStyles.put(sCurrentOutlineStyle,sNames);
            StyleWithProperties style1 = ofr.getPresentationStyle(sCurrentOutlineStyle);
            if (style1!=null) {
                String sCurrentOutlineStyle1 = sCurrentOutlineStyle;
                if (style1.isAutomatic()) { sCurrentOutlineStyle1 = style1.getParentName(); }
                sNames[1] = sCurrentOutlineStyle1;
                String sBaseName = sCurrentOutlineStyle1.substring(0,sCurrentOutlineStyle1.length()-1);
                for (int i=2; i<10; i++) {
                    String sName = sBaseName + Integer.toString(i);
                    StyleWithProperties style = ofr.getPresentationStyle(sName);
                    if (style!=null && style.getParentName().equals(sNames[i-1])) {
                        sNames[i] = sName;
                    }
                    else {
                        break;
                    }
                }
                sNames[1] = null;
            }
        }
    }
	
    public void exitOutline() {
        sCurrentOutlineStyle = null;
    }
	
    public void applyOutlineStyle(int nLevel, StyleInfo info) {
        if (2<=nLevel && nLevel<=9 && sCurrentOutlineStyle!=null) {
            if (outlineStyles.containsKey(sCurrentOutlineStyle)) {
            	String name = outlineStyles.get(sCurrentOutlineStyle)[nLevel];
            	outlineStyleNames.addName(name);
              info.sClass = "outline"+outlineStyleNames.getName(name);
            }
        }
    }
	
    
	
}
